@@ -11,6 +11,11 @@ def count_records((key, meta, rec)):
     global count
     count += 1
 
+def count_records_false((key, meta, rec)):
+    global count
+    count += 1
+    return False
+
 def digest_only(key):
     return (key[0], key[1], None, key[3])
 
@@ -41,8 +46,6 @@ class KVTestCase(unittest.TestCase):
             "b": bytearray("efgh","utf-8"),
             "l": [1357, "aceg", bytearray("aceg","utf-8"), [1,3,5,7], {"a": 1, "c": 3, "e": 5, "g": 7}],
             "m": {"i": 2468, "s": "bdfh", "l": [2468, "bdfh", bytearray("bdfh","utf-8")], "m": {"b": 2, "d": 4, "f": 6, "h": 8}},
-            "n": {"t" : u"\ud83d\ude04"},
-            "o": u"\ud83d\ude04"
         }
 
         # create the record
@@ -51,7 +54,7 @@ class KVTestCase(unittest.TestCase):
 
         # ensure existence
         (key, meta) = self.client.exists(key)
-        self.assertIsNotNone(meta, 'record should exist')
+        self.assertTrue(meta is not None)
 
         # count records
         count = 0
@@ -79,14 +82,13 @@ class KVTestCase(unittest.TestCase):
 
         # ensure not existent
         (key, meta) = self.client.exists(key)
-        self.assertIsNone(meta, 'record should not exist')
+        self.assertTrue(meta is None)
 
         # count records
         count = 0
         self.client.scan("test","unittest").foreach(count_records)
         self.assertEqual(count, 0, 'set should be empty')
 
-
     def test_2(self):
         '''
         Using a single key, with digest only.
@@ -106,7 +108,11 @@ class KVTestCase(unittest.TestCase):
             "s": "abcd",
             "b": bytearray("efgh","utf-8"),
             "l": [1357, "aceg", bytearray("aceg","utf-8"), [1,3,5,7], {"a": 1, "c": 3, "e": 5, "g": 7}],
-            "m": {"i": 2468, "s": "bdfh", "l": [2468, "bdfh", bytearray("bdfh","utf-8")], "m": {"b": 2, "d": 4, "f": 6, "h": 8}}
+            "m": {"i": 2468, "s": "bdfh", "l": [2468, "bdfh",
+                bytearray("bdfh","utf-8")], "m": {"b": 2, "d": 4, "f": 6, "h":
+                    8}},
+            'a': {u'aa': u'11'}, 
+            'k': {u'kk': u'22'}
         }
         
         # create the record
@@ -115,7 +121,7 @@ class KVTestCase(unittest.TestCase):
 
         # ensure existence
         (key, meta) = self.client.exists(key)
-        self.assertIsNotNone(meta, 'record should exist')
+        self.assertTrue(meta is not None)
 
         # count records
         count = 0
@@ -143,13 +149,48 @@ class KVTestCase(unittest.TestCase):
 
         # ensure not existent
         (key, meta) = self.client.exists(digest_only(key))
-        self.assertIsNone(meta, 'record should not exist')
+        self.assertTrue(meta is None)
 
         # count records
         count = 0
         self.client.scan("test","unittest").foreach(count_records)
         self.assertEqual(count, 0, 'set should be empty')
 
+    def test_3(self):
+        """
+        Using multiple keys
+        """
+        from aerospike import predicates as p
+        from time import sleep
+        global count
 
+        for i in xrange(2):
+            key = ('test', 'unittest', i)
+            rec = {
+                'name' : 'name%s' % (str(i)),
+                'addr' : 'name%s' % (str(i)),
+                'age'  : i,
+                'no'   : i
+            }
+            self.client.put(key, rec)
+
+        self.client.index_integer_create({}, 'test', 'unittest', 'age', 'age_index')
+        sleep(1)
+
+        query = self.client.query('test', 'unittest')
+
+        query.select("name", "age")
+        count = 0
+        query.where(p.between('age', 1, 3))
+
+        query.foreach(count_records_false)
+
+        self.assertEqual(count, 1, "foreach failed")
+
+        for i in xrange(2):
+            key = ('test', 'unittest', i)
+            self.client.remove(key)
+
+        self.client.index_remove({}, 'test', 'age_index');
 
 suite = unittest.TestLoader().loadTestsFromTestCase(KVTestCase)
